@@ -1,5 +1,5 @@
 /* ldapdelete.c - simple program to delete an entry using LDAP */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.30.2.15 2000/09/20 19:32:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.30.2.16 2000/11/06 16:49:43 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -376,7 +376,7 @@ main( int argc, char **argv )
 		{
 			char* p;
 
-			for( p = optarg; *p == '\0'; p++ ) {
+			for( p = optarg; *p != '\0'; p++ ) {
 				*p = '\0';
 			}
 		}
