@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.37.2.15 2000/09/20 19:32:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.37.2.17 2000/11/06 16:49:44 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -504,7 +504,7 @@ main( int argc, char **argv )
 		{
 			char* p;
 
-			for( p = optarg; *p == '\0'; p++ ) {
+			for( p = optarg; *p != '\0'; p++ ) {
 				*p = '\0';
 			}
 		}
@@ -717,7 +717,7 @@ main( int argc, char **argv )
 	}
 
 	if (version == -1 ) {
-		version = 3;
+		version = LDAP_VERSION3;
 	}
 
 	if( ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &version )
@@ -1291,7 +1291,9 @@ static int print_result(
 	return err;
 }
 
-void print_ctrls( LDAPControl **ctrls ) {
+static void print_ctrls(
+	LDAPControl **ctrls )
+{
 	int i;
 	for(i=0; ctrls[i] != NULL; i++ ) {
 		/* control: OID criticality base64value */
