@@ -327,6 +327,10 @@ int ldap_pvt_sasl_install( Sockbuf *sb, void *ctx_arg )
 	if ( !ber_sockbuf_ctrl( sb, LBER_SB_OPT_HAS_IO,
 			&ldap_pvt_sockbuf_io_sasl ) )
 	{
+#ifdef LDAP_DEBUG
+		ber_sockbuf_add_io( sb, &ber_sockbuf_io_debug,
+			LBER_SBIOD_LEVEL_APPLICATION, (void *)"sasl_" );
+#endif
 		ber_sockbuf_add_io( sb, &ldap_pvt_sockbuf_io_sasl,
 			LBER_SBIOD_LEVEL_APPLICATION, ctx_arg );
 	}
@@ -403,7 +407,7 @@ ldap_int_sasl_open(
 	assert( lc->lconn_sasl_ctx == NULL );
 
 	if ( host == NULL ) {
-		ld->ld_errno = LDAP_UNAVAILABLE;
+		ld->ld_errno = LDAP_LOCAL_ERROR;
 		return ld->ld_errno;
 	}
 
@@ -482,13 +486,15 @@ ldap_int_sasl_bind(
 
 	if ( sd == AC_SOCKET_INVALID ) {
  		/* not connected yet */
- 		int rc = ldap_open_defconn( ld );
-  
+ 		int rc;
+
+		rc = ldap_open_defconn( ld );
 		if( rc < 0 ) return ld->ld_errno;
+
 		ber_sockbuf_ctrl( ld->ld_sb, LBER_SB_OPT_GET_FD, &sd );
 
 		if( sd == AC_SOCKET_INVALID ) {
-			ld->ld_errno = LDAP_UNAVAILABLE;
+			ld->ld_errno = LDAP_LOCAL_ERROR;
 			return ld->ld_errno;
 		}
 	}   
@@ -496,7 +502,7 @@ ldap_int_sasl_bind(
 	ctx = ld->ld_defconn->lconn_sasl_ctx;
 
 	if( ctx == NULL ) {
-		ld->ld_errno = LDAP_UNAVAILABLE;
+		ld->ld_errno = LDAP_LOCAL_ERROR;
 		return ld->ld_errno;
 	}
 
@@ -648,6 +654,35 @@ ldap_int_sasl_bind(
 	return rc;
 }
 
+int
+ldap_int_sasl_external(
+	LDAP *ld,
+	const char * authid,
+	ber_len_t ssf )
+{
+	int sc;
+	sasl_conn_t *ctx = ld->ld_defconn->lconn_sasl_ctx;
+	sasl_external_properties_t extprops;
+    
+	if ( ctx == NULL ) {
+		return LDAP_LOCAL_ERROR;
+	}
+    
+	memset( &extprops, '\0', sizeof(extprops) );
+	extprops.ssf = ssf;
+	extprops.auth_id = (char *) authid;
+    
+	sc = sasl_setprop( ctx, SASL_SSF_EXTERNAL,
+		(void *) &extprops );
+    
+	if ( sc != SASL_OK ) {
+		return LDAP_LOCAL_ERROR;
+	}
+
+	return LDAP_SUCCESS;
+}
+
+
 int ldap_pvt_sasl_secprops(
 	const char *in,
 	sasl_security_properties_t *secprops )
@@ -947,4 +982,12 @@ ldap_int_sasl_bind(
 	LDAP_SASL_INTERACT_PROC *interact,
 	void * defaults )
 { return LDAP_NOT_SUPPORTED; }
+
+int
+ldap_int_sasl_external(
+	LDAP *ld,
+	const char * authid,
+	ber_len_t ssf )
+{ return LDAP_SUCCESS; }
+
 #endif /* HAVE_CYRUS_SASL */
