@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.30.2.9 2000/10/02 17:22:36 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.30.2.12 2000/10/30 17:37:17 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -351,10 +351,11 @@ find_connection( LDAP *ld, LDAPURLDesc *srv, int any )
 	for ( lc = ld->ld_conns; lc != NULL; lc = lc->lconn_next ) {
 		for ( ls = srv; ls != NULL; ls = ls->lud_next ) {
 			if ( lc->lconn_server->lud_host != NULL &&
-			    ls->lud_host != NULL && strcasecmp(
-			    ls->lud_host, lc->lconn_server->lud_host ) == 0
+				*lc->lconn_server->lud_host != '\0' &&
+			    ls->lud_host != NULL && *ls->lud_host != '\0' &&
+				strcasecmp( ls->lud_host, lc->lconn_server->lud_host ) == 0
 			    && ls->lud_port == lc->lconn_server->lud_port ) {
-				return( lc );
+				return lc;
 			}
 			if ( !any ) {
 				break;
@@ -362,7 +363,7 @@ find_connection( LDAP *ld, LDAPURLDesc *srv, int any )
 		}
 	}
 
-	return( NULL );
+	return NULL;
 }
 
 
@@ -575,13 +576,14 @@ ldap_free_request( LDAP *ld, LDAPRequest *lr )
  *  (IN) lr = LDAP Request structure
  *  (IN) refs = array of pointers to referral strings that we will chase
  *              The array will be free'd by this function when no longer needed
+ *  (IN) sref != 0 if following search reference
  *  (OUT) errstrp = Place to return a string of referrals which could not be followed
  *  (OUT) hadrefp = 1 if sucessfully followed referral
  *
  * Return value - number of referrals followed
  */
 int
-ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, char **errstrp, int *hadrefp )
+ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char **errstrp, int *hadrefp )
 {
 	char		*unfollowed;
 	int			 unfollowedcnt = 0;
@@ -632,6 +634,13 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, char **errstrp,
 			goto done;
 		}
 
+		if( srv->lud_crit_exts ) {
+			/* we do not support any extensions */
+			ld->ld_errno = LDAP_NOT_SUPPORTED;
+			rc = -1;
+			goto done;
+		}
+
 		/* treat ldap://hostpart and ldap://hostpart/ the same */
 		if ( srv->lud_dn && srv->lud_dn[0] == '\0' ) {
 			LDAP_FREE( srv->lud_dn );
@@ -687,6 +696,11 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, char **errstrp,
 		 * Note: In the future we also need to replace the filter if one
 		 * was provided with the search reference
 		 */
+
+		/* For references we don't want old dn if new dn empty */
+		if ( sref && srv->lud_dn == NULL )
+			srv->lud_dn = LDAP_STRDUP( "" );
+
 		if (( ber = re_encode_request( ld, origreq->lr_ber,
 			    ++ld->ld_msgid, &srv->lud_dn, &rinfo.ri_request )) == NULL ) {
 			ld->ld_errno = LDAP_ENCODING_ERROR;
