@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.8.2.10 2000/10/02 17:22:36 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.8.2.13 2000/11/04 21:38:28 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -361,6 +361,11 @@ sb_tls_ctrl( Sockbuf_IO_Desc *sbiod, int opt, void *arg )
 	if ( opt == LBER_SB_OPT_GET_SSL ) {
 		*((SSL **)arg) = p->ssl;
 		return 1;
+
+	} else if ( opt == LBER_SB_OPT_DATA_READY ) {
+		if( SSL_pending( p->ssl ) > 0 ) {
+			return 1;
+		}
 	}
 	
 	return LBER_SBIOD_CTRL_NEXT( sbiod, opt, arg );
@@ -875,6 +880,20 @@ ldap_pvt_tls_start ( LDAP *ld, Sockbuf *sb, void *ctx_arg )
 	 * certificate....
 	 */
 
+
+	{
+		void *ssl;
+		const char *authid;
+		ber_len_t ssf;
+
+		/* we need to let SASL know */
+		ssl = (void *) ldap_pvt_tls_sb_handle( sb );
+		ssf = ldap_pvt_tls_get_strength( ssl );
+		authid = ldap_pvt_tls_get_peer( ssl );
+
+		(void) ldap_int_sasl_external( ld, authid, ssf );
+	}
+
 	return LDAP_SUCCESS;
 }
 
@@ -1010,7 +1029,7 @@ tls_seed_PRNG( const char *randfile )
 
 	if (randfile == NULL) {
 		Debug( LDAP_DEBUG_ANY,
-			"TLS: Use configuration file or $RANDFILE to define seed PRNG",
+			"TLS: Use configuration file or $RANDFILE to define seed PRNG\n",
 			0, 0, 0);
 		return -1;
 	}
@@ -1019,7 +1038,7 @@ tls_seed_PRNG( const char *randfile )
 
 	if (RAND_status() == 0) {
 		Debug( LDAP_DEBUG_ANY,
-			"TLS: PRNG not been seeded with enough data",
+			"TLS: PRNG not been seeded with enough data\n",
 			0, 0, 0);
 		return -1;
 	}
