@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.29.2.7 2000/09/30 16:38:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.29.2.10 2000/10/30 17:40:18 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -153,8 +153,7 @@ skip_url_prefix(
 	}
 
 	/* skip leading "URL:" (if any) */
-	if ( strncasecmp( p, LDAP_URL_URLCOLON, LDAP_URL_URLCOLON_LEN ) == 0 )
-	{
+	if ( strncasecmp( p, LDAP_URL_URLCOLON, LDAP_URL_URLCOLON_LEN ) == 0 ) {
 		p += LDAP_URL_URLCOLON_LEN;
 	}
 
@@ -278,6 +277,7 @@ ldap_url_parse( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	ludp->lud_filter = NULL;
 	ludp->lud_scope = LDAP_SCOPE_BASE;
 	ludp->lud_filter = NULL;
+	ludp->lud_exts = NULL;
 
 	ludp->lud_scheme = LDAP_STRDUP( scheme );
 
@@ -517,6 +517,11 @@ ldap_url_parse( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 
 	for( i=0; ludp->lud_exts[i] != NULL; i++ ) {
 		ldap_pvt_hex_unescape( ludp->lud_exts[i] );
+
+		if( *ludp->lud_exts[i] == '!' ) {
+			/* count the number of critical extensions */
+			ludp->lud_crit_exts++;
+		}
 	}
 
 	if( i == 0 ) {
@@ -878,11 +883,20 @@ ldap_url_search( LDAP *ld, LDAP_CONST char *url, int attrsonly )
 	BerElement	*ber;
 	LDAPreqinfo  bind;
 
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+
 	if ( ldap_url_parse( url, &ludp ) != 0 ) {
 		ld->ld_errno = LDAP_PARAM_ERROR;
 		return( -1 );
 	}
 
+	if( ludp->lud_crit_exts ) {
+		/* we don't support any extension (yet) */
+		ld->ld_errno = LDAP_NOT_SUPPORTED;
+		return( -1 );
+	}
+
 	ber = ldap_build_search_req( ld, ludp->lud_dn, ludp->lud_scope,
 	    ludp->lud_filter, ludp->lud_attrs, attrsonly, NULL, NULL,
 		-1, -1 );
@@ -895,9 +909,7 @@ ldap_url_search( LDAP *ld, LDAP_CONST char *url, int attrsonly )
 		bind.ri_url = (char *)url;
 		err = ldap_send_server_request(
 					ld, ber, ld->ld_msgid, NULL,
-					(ludp->lud_host != NULL || ludp->lud_port != 0)
-						? ludp : NULL,
-					NULL, &bind );
+					NULL, NULL, &bind );
 	}
 
 	ldap_free_urldesc( ludp );
