@@ -1,5 +1,5 @@
 /* aclparse.c - routines to parse and check acl's */
-/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.24.2.8 2000/09/05 17:54:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.24.2.9 2000/10/11 02:43:57 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -508,7 +508,9 @@ parse_acl(
 					}
 
 					if( !is_at_syntax( b->a_group_at->ad_type,
-						SLAPD_DN_SYNTAX ) )
+						SLAPD_DN_SYNTAX ) &&
+					    !is_at_syntax( b->a_group_at->ad_type,
+						SLAPD_NAMEUID_SYNTAX ) )
 					{
 						fprintf( stderr,
 							"%s: line %d: group \"%s\": inappropriate syntax: %s\n",
