@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.42.2.9 2000/09/20 19:32:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.42.2.11 2000/10/30 18:16:08 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -427,9 +427,12 @@ be_db_close( void )
 }
 
 Backend *
-select_backend( const char * dn )
+select_backend(
+	const char * dn,
+	int manageDSAit )
 {
 	int	i, j, len, dnlen;
+	Backend *be = NULL;
 
 	dnlen = strlen( dn );
 	for ( i = 0; i < nbackends; i++ ) {
@@ -443,13 +446,23 @@ select_backend( const char * dn )
 			}
 
 			if ( strcmp( backends[i].be_nsuffix[j],
-			    dn + (dnlen - len) ) == 0 ) {
-				return( &backends[i] );
+			    dn + (dnlen - len) ) == 0 )
+			{
+				if( be == NULL ) {
+					be = &backends[i];
+
+					if( manageDSAit && len == dnlen ) {
+						continue;
+					}
+				} else {
+					be = &backends[i];
+				}
+				return be;
 			}
 		}
 	}
 
-	return( NULL );
+	return be;
 }
 
 int
@@ -513,7 +526,7 @@ be_isroot_pw( Backend *be,
 		return 0;
 	}
 
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_PASSWD )
+#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
 	ldap_pvt_thread_mutex_lock( &passwd_mutex );
 #ifdef SLAPD_SPASSWD
 	lutil_passwd_sasl_conn = conn->c_sasl_context;
@@ -522,7 +535,7 @@ be_isroot_pw( Backend *be,
 
 	result = lutil_passwd( &be->be_root_pw, cred, NULL );
 
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_PASSWD )
+#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
 #ifdef SLAPD_SPASSWD
 	lutil_passwd_sasl_conn = NULL;
 #endif
@@ -835,7 +848,7 @@ backend_group(
 	if( strcmp( target->e_ndn, gr_ndn ) != 0 ) {
 		/* we won't attempt to send it to a different backend */
 		
-		be = select_backend(gr_ndn);
+		be = select_backend(gr_ndn, 0);
 
 		if (be == NULL) {
 			return LDAP_NO_SUCH_OBJECT;
@@ -864,7 +877,7 @@ backend_attribute(
 	if( target == NULL || strcmp( target->e_ndn, e_ndn ) != 0 ) {
 		/* we won't attempt to send it to a different backend */
 		
-		be = select_backend(e_ndn);
+		be = select_backend(e_ndn, 0);
 
 		if (be == NULL) {
 			return LDAP_NO_SUCH_OBJECT;
