@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slappasswd.c,v 1.2.2.4 2000/08/26 17:06:18 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slappasswd.c,v 1.2.2.5 2000/11/06 16:49:44 kurt Exp $ */
 /*
  * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -56,6 +56,7 @@ main( int argc, char *argv[] )
 		switch (i) {
 		case 'h':	/* scheme */
 			scheme = strdup (optarg);
+			break;
 
 		case 's':	/* new password (secret) */
 			newpw = strdup (optarg);
@@ -63,7 +64,7 @@ main( int argc, char *argv[] )
 			{
 				char* p;
 
-				for( p = optarg; *p == '\0'; p++ ) {
+				for( p = optarg; *p != '\0'; p++ ) {
 					*p = '\0';
 				}
 			}
